/******************************************************************************
 * @file additional_sf_ops.c
 *
 * 		This file contains some functions that handles read file, write file and
 * delete file operations on the Serial Flash memory using the 'quad_spi' device
 * driver. It also records every operation on an XML file as a new node entry.
 * 		With the functions implemented here you can read a file from every
 * location in the memory, but writing and deleting a file can only be made at
 * the end of the previously written file.
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date     Changes
 * ----- ---- -------- -----------------------------------------------
 * 1.00a nagy 05/21/11 First release
 *
 * </pre>
 *
 *****************************************************************************/
/***************************** Include Files *********************************/

#include "additional_sf_ops.h"

/************************** Variable Definitions *****************************/

char * buf, * tmpBuf;
char *endPtr, *startPtr;
int fd, fdTmp, fsize;
int tmp, i=0;
int tmpCnt=0;

extern int verbose;
extern int addrCntWr;
extern int addrCntRd;

/************************** Function Definitions *****************************/
/******************************************************************************
 * Writes an entry to XML.
 *
 * @param	QSpiBaseAddr is the base address for the memory controller
 * @param	xmlFilename is the name of the XML file
 * @param	filename is the name of the file to be written
 * @param	location is the address from where the file will be written
 * @param	size is the size of the file
 *
 * @return	0 - error
 * 			1 - everything went ok
 *****************************************************************************/
//XXX: WriteXmlLastEntry
int WriteXmlLastEntry(unsigned long QSpiBaseAddr, const char *xmlFilename,
		const char *filename, const char *location, const char *size)
{
	int cpyRes;

	// check if the file already on MFS
	if(!mfs_exists_file((char *)xmlFilename))
	{
		cpyRes = CopyXmlFromSf(QSpiBaseAddr, xmlFilename);

		// copy the XML from Flash, if it exists on Flash else create one
		if(cpyRes == -1)
		{
			if(verbose) xil_printf("\nCreating XML file...");

			// create the file
			fd = mfs_file_open(xmlFilename, MFS_MODE_CREATE);

			// allocate space in memory to write the XML
			buf = (char *)malloc(160);
			memset(buf, 0, sizeof(buf));
			if(buf == NULL)
			{
				xil_printf("\nCannot allocate memory to create XML file");
				mfs_file_close(fd);
				free(buf);
				return 0;
			}

			// combine XML data into buf
			strcpy(buf, "<FILE_TRACKING>\r\n");
			strcat(buf, "    <FILE>\r\n");
			strcat(buf, "        <NAME>none</NAME>\r\n");
			strcat(buf, "        <LOCATION>0000000</LOCATION>\r\n");
			strcat(buf, "        <SIZE>0000000</SIZE>\r\n");
			strcat(buf, "    </FILE>\r\n");
			strcat(buf, "</FILE_TRACKING>");

			// write the file
			if(!mfs_file_write(fd, buf, strlen(buf)))
			{
				xil_printf("\nError writing XML file");
				mfs_file_close(fd);
				free(buf);
				return 0;
			}

			if(verbose) xil_printf("\nDone creating XML file");

			// close file
			mfs_file_close(fd);

			// free allocated memory
			free(buf);
		}
		else if(!cpyRes) // error
		{
			return 0;
		}
	}

//	fd = mfs_file_open("upload.xml", MFS_MODE_READ);
//	mfs_file_lseek(fd, 0, MFS_SEEK_SET);
//	fsize = mfs_file_lseek(fd, 0, MFS_SEEK_END);
//	xil_printf("\nfsize = %d", fsize);
//	char *hbuf = (char *)malloc(fsize);
//	memset(hbuf, 0, fsize);
//	mfs_file_read(fd, hbuf, fsize);
//	int g;
//	for(g=0; g<fsize; g++) {
//		xil_printf("\n%d val = %c (%02x hex)", g, hbuf[g], hbuf[g]);
//	}
//	free(hbuf);
//	mfs_file_close(fd);

	if(verbose) xil_printf("\nAdding the new entry");

	// open the XML file for writing
	fd = mfs_file_open(xmlFilename, MFS_MODE_WRITE);

	if(fd == -1)
	{
		xil_printf("\nError opening file for writing new entry");
		return 0;
	}

	// get XML file size
	mfs_file_lseek(fd, 0, MFS_SEEK_SET);
	fsize = mfs_file_lseek(fd, 0, MFS_SEEK_END);

	// allocate memory for the strings
	buf = (char *)malloc(170);
	tmpBuf = (char *)malloc(60);
	memset(buf, 0, sizeof(buf));
	memset(tmpBuf, 0, sizeof(tmpBuf));
	if((buf == NULL)||(tmpBuf == NULL))
	{
		xil_printf("\nUnable to allocate memory for the new XML entry");
		mfs_file_close(fd);
		free(tmpBuf);
		free(buf);
		return 0;
	}

	// create an entry
	strcpy(buf, "\r\n");
	strcat(buf, "    <FILE>\r\n");
	sprintf(tmpBuf, "        <NAME>%s</NAME>\r\n", filename);
	strcat(buf, tmpBuf);
	sprintf(tmpBuf, "        <LOCATION>%s</LOCATION>\r\n",
			(cpyRes == -1) ? "0000000" : location);
	strcat(buf, tmpBuf);
	sprintf(tmpBuf, "        <SIZE>%s</SIZE>\r\n", size);
	strcat(buf, tmpBuf);
	strcat(buf, "    </FILE>\r\n");
	strcat(buf, "</FILE_TRACKING>");

	// move pointer to before the '</FILE_TRACKING>' tag with -18 bytes
	mfs_file_lseek(fd, 0, MFS_SEEK_SET);
	mfs_file_lseek(fd, /*((cpyRes == -1) ? -18 : -19)*/-18, MFS_SEEK_END);

	// write the entry to file
	if(!mfs_file_write(fd, buf, strlen(buf)))
	{
		xil_printf("\nError writing to XML the new entry");
		mfs_file_close(fd);
		free(tmpBuf);
		free(buf);
		return 0;
	}

//	int g;
//	for(g=0; g<strlen(buf); g++) {
//		xil_printf("\n%d val = %c (%02x hex)", g, buf[g], buf[g]);
//	}

	// close the file and free-up memory
	if(!mfs_file_close(fd))
	{
		xil_printf("\nError closing file after added new entry to it, fd = %d", fd);
		return 0;
	}

	// free allocated memory
	free(buf);
	free(tmpBuf);

	if(verbose) xil_printf("\nDone writing to XML new entry");

	// now copy file back to Flash and erase it
//	if(!WriteXmlToSf(QSpiBaseAddr, xmlFilename))
//	{
//		return 0;
//	}

	return 1;
}

/******************************************************************************
 * Deletes the last entry of XML.
 *
 * @param	QSpiBaseAddr is the base address for the memory controller
 * @param	xmlFilename is the name of the XML file
 *
 * @return	0 - error
 * 			1 - everything went ok
 *****************************************************************************/
//XXX: DeleteXmlLastEntry
int DeleteXmlLastEntry(unsigned long QSpiBaseAddr, const char *xmlFilename,
		const char *filename)
{
	int offset, i=0;

	// copy XML from Flash and return if none
	int cpyRes = CopyXmlFromSf(QSpiBaseAddr, xmlFilename);
	if((cpyRes == -1)||(!cpyRes))
	{
		return 0;
	}

	// open XML file for reading
	fd = mfs_file_open(xmlFilename, MFS_MODE_READ);

	// get the size of the file
	mfs_file_lseek(fd, 0, MFS_SEEK_SET);
	fsize = mfs_file_lseek(fd, 0, MFS_SEEK_END);

	// and calculate offset (total size of last entry)
	offset = strlen(filename) + 134;

	// subtract the offset from fsize
	fsize -= offset;

	// open temporary file for creating and writing
	fdTmp = mfs_file_open("tmp.xml", MFS_MODE_CREATE);

	// allocate only "a safe" 512 bytes
	buf = (char *)malloc(257);
	memset(buf, 0, sizeof(buf));

	// read chunks of 256 bytes and copy them to the temporary file
	// this way the memory remains "safe"
	i = 0;

	while((i+256)<fsize)
	{
		tmp = mfs_file_read(fd, buf, 256);
		if(tmp != 256)
		{
			xil_printf("\nError reading from XML, %d bytes "
					"read of 256", tmp);
			mfs_file_close(fd);
			mfs_file_close(fdTmp);
			mfs_delete_file((char *)xmlFilename);
			mfs_delete_file("tmp.xml");
			free(buf);
			return 0;
		}
		if(!mfs_file_write(fdTmp, buf, 256))
		{
			xil_printf("\nError writing data to temporary file");
			mfs_file_close(fd);
			mfs_file_close(fdTmp);
			mfs_delete_file((char *)xmlFilename);
			mfs_delete_file("tmp.xml");
			free(buf);
			return 0;
		}
		memset(buf, 0, sizeof(buf));
		i += 256;
	}
	if((fsize-i) >= 1)
	{

		tmp = mfs_file_read(fd, buf, fsize-i);
		if(tmp != fsize-i)
		{
			xil_printf("\nError reading from XML (remaining), "
					"%d bytes read of %d", tmp, fsize-i);
			mfs_file_close(fd);
			mfs_file_close(fdTmp);
			mfs_delete_file((char *)xmlFilename);
			mfs_delete_file("tmp.xml");
			free(buf);
			return 0;
		}
		if(!mfs_file_write(fdTmp, buf, fsize-i))
		{
			xil_printf("\nError writing data to temporary file");
			mfs_file_close(fd);
			mfs_file_close(fdTmp);
			mfs_delete_file((char *)xmlFilename);
			mfs_delete_file("tmp.xml");
			free(buf);
			return 0;
		}
		memset(buf, 0, sizeof(buf));
	}

	//and write the last line '</FILE_TRACKING>'
	if(!mfs_file_write(fdTmp, "\r\n</FILE_TRACKING>", 18))
	{
		mfs_file_close(fd);
		mfs_file_close(fdTmp);
		mfs_delete_file((char *)xmlFilename);
		mfs_delete_file("tmp.xml");
		free(buf);
		xil_printf("\nError writing the last line to temporary file");
		return 0;
	}

	//free the buffer
	free(buf);

	// close the files
	mfs_file_close(fd);
	mfs_file_close(fdTmp);

	// then delete the XML
	if(!mfs_delete_file((char *)xmlFilename))
	{
		xil_printf("\nError deleting XML file");
		return 0;
	}

	// rename the temporary file to XML's name
	if(!mfs_rename_file("tmp.xml", (char *)xmlFilename))
	{
		xil_printf("\nError renaming temporary file");
		return 0;
	}

	// write XML back to Flash
	if(!WriteXmlToSf(QSpiBaseAddr, xmlFilename, 0))
	{
		xil_printf("\nError writing XML to Flash");
		return 0;
	}

	return 1;
}

/******************************************************************************
 * Writes a chunk of maximum 256 bytes to the Serial Flash memory.
 *
 * @param	QSpiBaseAddr is the base address for the memory controller
 * @param	buf is the chunk of data to write (maximum 256 bytes)
 * @param	locationLastEntry is the start address of the last written file
 * @param	sizeLastEntry is the size of the last written file
 *
 * @return	0 - error
 * 			? - number of bytes written
 *****************************************************************************/
//XXX: WriteFileToSf
int WriteFileToSf(unsigned long QSpiBaseAddr, char *buf, const char *location,
		int size)
{
	int startAddress;

	// store start address for the writings
	// (according to the previously written file)
	startAddress = (int)htoi((char *)location);

//	if(verbose) xil_printf("\noldLocation = 0x%x, oldSize = 0x%x",
//			oldLocation, oldSize);

	// check if the chuck of data to be written exceeds Flash size
	if((startAddress + addrCntWr + size) >= FLASH_SIZE)
	{
		xil_printf("\nNo more memory left on Flash for writing");
		return 0;
	}

	// check if buf exceeds 256 bytes (page size for Serial Flash)
	if(size > 256)
	{
		xil_printf("\nBuffer exceeds maximum write size of 256 bytes");
		return 0;
	}

	// write the chunk of data to Serial Flash
	Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, startAddress + addrCntWr,
			size, (unsigned char *)buf);

	if(verbose) xil_printf("\nWritten %d bytes to 0x%x", size,
			startAddress + addrCntWr);

	// counter for the address
	addrCntWr += size;

	return size;
}

/******************************************************************************
 * Reads a chunk of maximum 256 bytes from the Serial Flash memory from any
 * given start address.
 *
 * @param	QSpiBaseAddr is the base address for the memory controller
 * @param	buf is the chunk of data to write (maximum size of 256 bytes)
 * @param	location is the start address of the file to be read
 * @param	size is the number of bytes to read
 *
 * @return	0 - error
 * 			? - number of bytes read
 *****************************************************************************/
//XXX: ReadFileFromSf
int ReadFileFromSf(unsigned long QSpiBaseAddr, char *buf, const char *location,
		const char *size)
{
	int newLocation;
	int newSize;

	// convert character arrays to integers
	newLocation = (int)htoi((char *)location);
	newSize = (int)htoi((char *)size);

	if(verbose) xil_printf("\nnewLocation = 0x%x, newSize = 0x%x",
			newLocation, newSize);

	// read batches of 256 bytes
	if(addrCntRd<(newSize/256))
	{
		Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, newLocation+(addrCntRd*256),
				256, 8, (unsigned char *)buf);
		if(verbose) xil_printf("\nRead 256 bytes from 0x%x",
				newLocation+(addrCntRd*256));
		addrCntRd++;

		return 256;
	}
	// read the remaining bytes, if any
	else if(newSize%256 >= 1)
	{
		Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, newLocation+(addrCntRd*256),
				newSize%256, 8, (unsigned char *)buf);
		if(verbose) xil_printf("\nRead %d bytes from 0x%x", newSize%256,
						newLocation+(addrCntRd*256));
	}

	// return the number of read bytes
	return (newSize%256);
}

/******************************************************************************
 * Deletes a file from Serial Flash memory given by its size and start address.
 *
 * @param	QSpiBaseAddr is the base address for the memory controller
 * @param	location is the start address of the file
 * @param	size is the size of the file
 *
 * @return	0 - error
 * 			1 - everything went ok
 *****************************************************************************/
//XXX: DeleteFileFromSf
int DeleteFileFromSf(unsigned long QSpiBaseAddr, const char *location,
		const char *size)
{
	int addrRez, startAddr;
	int loc, siz, j=0;
	char *fbuf;
	unsigned char dRead[256];

	// convert character arrays to integers
	loc = (int)htoi((char *)location);
	siz = (int)htoi((char *)size);

	// check if the address is a sector start address
	addrRez = loc % FLASH_SECTOR_SIZE;

	// store start address of the sector
	startAddr = loc - addrRez;

	// allocate memory to read-back data on the memory
	fbuf = (char *)malloc(addrRez);
	memset(fbuf, 0, addrRez);
	if(fbuf == NULL)
	{
		xil_printf("\nCannot allocate memory for deleting file");
		return 0;
	}

	// set the pointer to the start
	startPtr = fbuf;

	if(verbose) xil_printf("\nFLASH: Reading back %d bytes of data from "
			"address 0x%x", addrRez, startAddr);

	if(addrRez)
	{
		// the residue is bigger than a page size
		if(addrRez > 256)
		{
			// read chunks of 256 bytes
			for(i=0; i<(addrRez/256); i++)
			{
				Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, (startAddr+(i*256)),
						256, 8, dRead);
				fbuf = (char *)dRead;
				fbuf += 256;
			}
			// then read the remaining bytes
			if(addrRez%256 >= 1)
			{
				Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, (startAddr+(i*256)),
						(addrRez%256), 8, dRead);
				fbuf = (char *)dRead;
			}
		}
		// if not then perform a single page read
		else
		{
			Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, startAddr, addrRez, 8, dRead);
			fbuf = (char *)dRead;
		}
	}

	if(verbose) xil_printf("\nFLASH: Deleting the file (%d bytes) "
			"from sector start address 0x%x of sector", siz, startAddr);

	// erase now the sector(s)
	while(j < siz)
	{
		Sector_Erase(QSpiBaseAddr, 4, 0, startAddr+j);
		j += FLASH_SECTOR_SIZE;
	}

	if(verbose) xil_printf("\nFLASH: Writing back %d bytes of data to "
			"address 0x%x", addrRez, startAddr);

	// write back the data read, if any
	if(addrRez)
	{
		// if bigger than 256 then multiple writes need to be performed
		if(addrRez > 256)
		{
			// write chunks of 256 bytes of data
			for(i=0; i<(addrRez/256); i++)
			{
				Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, (startAddr+(i*256)),
						256, (unsigned char *)fbuf);
				fbuf += 256;
			}
			// write the remaining bytes of data
			if(addrRez%256)
			{
				Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, (startAddr+(i*256)),
						addrRez%256, (unsigned char *)fbuf);
			}
		}
		// if not then a single one if enough
		else
		{
			Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, startAddr, addrRez,
					(unsigned char *)fbuf);
		}
	}

	//free-up allocated memory
	free(fbuf);

	return 1;
}

/******************************************************************************
 * Reads the XML stored at address 0xFFC000+2 in the Serial Flash, saves it to
 * RAM using MFS. The first two bytes specify the XML file's size.
 *
 * @param	QSpiBaseAddr is the base address for the memory controller
 * @param	xmlFilename is the name of the XML
 *
 * @return	0 - error
 * 			1 - everything went ok
 * 		   -1 - no XML present on Serial Flash memory
 *****************************************************************************/
//XXX: CopyXmlFromSf
int CopyXmlFromSf(unsigned long QSpiBaseAddr, const char *xmlFilename)
{
	int xmlSize = 0;
	unsigned char dRead[2];

	// read the first 2 bytes and determine XML file size
	Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, XML_START_ADDR, 2, 8, dRead);

	// store the size
	xmlSize = (dRead[1]<<8)|dRead[0];
	if(verbose) xil_printf("\nXML file size: %d bytes (%04x hexa)",
			xmlSize, xmlSize);

	// if file present in memory
	if((xmlSize != 0xFFFF)&&(xmlSize != 0))
	{
		int j=0;
		if(verbose) xil_printf("\nXML present in Flash memory");

		// allocate memory space for the buffer
		buf = (char *)malloc(260);
		memset(buf, 0, sizeof(buf));
		if(buf == NULL)
		{
			xil_printf("\nCannot allocate buffer space in memory");
			return 0;
		}

		if(verbose) xil_printf("\nCreating XML file and copying %d bytes of XML "
				"data from Flash, from address 0x%x...", xmlSize, XML_START_ADDR+2);

		// open and create an XML file on the local MFS
		fd = mfs_file_open(xmlFilename, MFS_MODE_CREATE);

		// read the integer parts of 256 bytes of the XML
		for(j=0; j<(xmlSize/256); j++)
		{
			Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, XML_START_ADDR+2+(j*256),
					256, 8, (unsigned char *)buf);
			if(!mfs_file_write(fd, buf, 256))
			{
				if(verbose) xil_printf("\nError writing data "
						"to RAM XML from Flash");
				return 0;
			}
		}
		// read the remaining bytes, if any
		if(xmlSize%256 >= 1)
		{
			Quad_Output_Fast_Read(QSpiBaseAddr, 4, 0, XML_START_ADDR+2+(j*256),
					xmlSize%256, 8, (unsigned char *)buf);
			if(!mfs_file_write(fd, buf, xmlSize%256))
			{
				if(verbose) xil_printf("\nError writing data "
						"from Flash to RAM XML");
				return 0;
			}
		}

		// close the file
		mfs_file_close(fd);

		// free allocated memory space
		free(buf);

		if(verbose) xil_printf("\nDone copying XML data from Flash");
	}
	else // no XML file on Flash memory
	{
		xil_printf("\nNo XML data on Serial Flash");
		return -1;
	}

	return 1;
}

/******************************************************************************
 * Reads the XML stored at on the current MFS and write's it to Serial Flash at
 * address 0xFF0000+2, storing at address 0xFF0000 the XML's size.
 *
 * @param	QSpiBaseAddr is the base address for the memory controller
 * @param	xmlFilename is the name of the XML
 *
 * @return	0 - error
 * 			1 - everything went ok
 *
 * @note	If calling this function after added a new entry, make sure you
 * 			decrement fsize with 17 (fsize-17) to avoid writing NULL character
 * 			garbage at the end of file.
 *****************************************************************************/
//XXX: WriteXmlToSf
int WriteXmlToSf(unsigned long QSpiBaseAddr, const char *xmlFilename, int decSize)
{
	int j=0;

	// open the XML file
	fd = mfs_file_open(xmlFilename, MFS_MODE_READ);

	// get the size of XML
	fsize = mfs_file_lseek(fd, 0, MFS_SEEK_END);

	if(decSize) fsize -= 18;

	// allocate memory space for the buffer
	buf = (char *)malloc(260);
	memset(buf, 0, sizeof(buf));
	if(buf == NULL)
	{
		xil_printf("\nCannot allocate memory to write XML to Flash.");
		mfs_file_close(fd);
		free(buf);
		return 0;
	}

	if(verbose) xil_printf("\nFLASH: erasing the last sector from "
			"address 0x%x...", XML_START_ADDR);

	// erase the last sector
	Sector_Erase(QSpiBaseAddr, 4, 0, XML_START_ADDR);

	if(verbose) xil_printf("\nFLASH: done erasing.");
	if(verbose) xil_printf("\nFLASH: writing the XML data...");

	// now write the XML file
	for(j=0; j<fsize/256; j++)
	{
		if(j==0)
		{
			buf[0] = fsize&0xFF;
			buf[1] = (fsize>>8)&0xFF;
			if(mfs_file_read(fd, buf+2, 254) != 254)
			{
				xil_printf("\nError reading (batches of 256) XML from MFS");
				mfs_file_close(fd);
				free(buf);
				return 0;
			}
			Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, XML_START_ADDR+(j*256),
					256, (unsigned char *)buf);
			memset(buf, 0, 260);
		}
		else
		{
			if(mfs_file_read(fd, buf, 256) != 256)
			{
				xil_printf("\nError reading (batches of 256) XML from MFS");
				mfs_file_close(fd);
				free(buf);
				return 0;
			}
			Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, XML_START_ADDR+(j*256),
					256, (unsigned char *)buf);
			memset(buf, 0, 260);
		}
	}
	if((fsize+2)%256 >= 1)
	{
		if(mfs_file_read(fd, buf, (fsize+2)%256) != (fsize+2)%256)
		{
			xil_printf("\nError reading -> 1 (remaining) XML "
					"from MFS (%d)", (fsize+2)%256);
			mfs_file_close(fd);
			free(buf);
			return 0;
		}
		Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, XML_START_ADDR+(j*256),
				(fsize+2)%256, (unsigned char *)buf);
		memset(buf, 0, 260);
	}
	else
	{
		if(mfs_file_read(fd, buf, 2) != 2)
		{
		    xil_printf("\nError reading -> 2 (remaining) XML from MFS");
		    mfs_file_close(fd);
		    free(buf);
		    return 0;
		}
		Quad_Input_Fast_Program(QSpiBaseAddr, 4, 0, XML_START_ADDR+(j*256),
		    	2, (unsigned char *)buf);
		memset(buf, 0, 260);
	}

	// close the file
	mfs_file_close(fd);

	// now delete it from local MFS
	if(!mfs_delete_file((char *)xmlFilename))
	{
		xil_printf("\nError deleting XML from MFS");
		mfs_file_close(fd);
		free(buf);
		return 0;
	}

	// free allocated memory space
	free(buf);

	if(verbose) xil_printf("\nFLASH: done writing XML data");

	return 1;
}

/******************************************************************************
 * Function to convert a hexadecimal string to long.
 *
 * @param	s is the string to be converted
 *
 * @return	0 - error
 * 			1 - converted long type
 *****************************************************************************/
long htoi(char s[])
{
	long deci=0, dig=0;
	int pos=0;

	// point to the string
    char *p = &s[strlen(s)-1];

    while(p >= s)
    {
        if((dig = hchartoi(*p, pos)) < 0 )
        {
            xil_printf("\nError converting htoi");
            return 0;
        }
        deci += dig;
        --p;
        ++pos;
    }

    return deci;
}

/******************************************************************************
 * Function to convert hexadecimal character to decimal.
 *
 * @param	hexdig is the hex character
 * @param	pos is the relative position to string
 *
 * @return	0 - error
 * 			1 - converted character
 *****************************************************************************/
long hchartoi(char hexdig, int pos)
{
    char hexdigits[] = "0123456789ABCDEF";
    long deci = 0;

    // point to the beginning of the 'hexdigits' array
    char *p = &hexdigits[0];

    while((*p != toupper(hexdig)) && (deci<16))
    {
        ++p;
        ++deci;
    }

    if(*p == toupper(hexdig))
    {
       for(i=0; i<pos; i++)
       {
           deci *= 16;
       }
       return deci;
    }

    return 0;
}

