/******************************************************************************
 * @file xquad_spi.c
 *
 * This file contains the implementation of the QuadSpi driver's basic
 * functionality. See xquad_spi.h for more information about the driver.
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date     Changes
 * ----- ---- -------- -----------------------------------------------
 * 1.00a nagy 04/04/11 First release
 *
 * </pre>
 *
 *****************************************************************************/

/***************************** Include Files *********************************/

#include "xquad_spi.h"

/************************** Variable Definitions *****************************/

Xuint32 Status;
Xuint32 start_command;
Xuint32 end_command;
Xuint32 i;

u8 Status_Reg;

extern unsigned char verbose;

/************************** Function Definitions *****************************/
/******************************************************************************
 * This function enables the WEL bit from the Write Latch. This must be done
 * prior to every Program, Erase or Write operation.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DivRate is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	Mode is the mode on which the communication between Quad-SPI IP
 * 			Core and Flash Memory is made: can be Extended Mode, Dual Mode,
 * 			Quad Mode.
 *
 * @return	None.
 *
 * @note	This function is available for all three modes.
 *****************************************************************************/
void Write_Enable(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate, Xuint32 Mode) {

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xFFFE0000&((DivRate<<17)|0xC001FFFF);
	end_command = 0x7FFE0000&((DivRate<<17)|0xC001FFFF);

	// Setting modes
	if(Mode == 2) { // cmd_mode = 2
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x02);
	}
	else if(Mode == 1) { // cmd_mode = 1
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x01);
	}
	else { // cmd_mode = 0
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x00);
	}

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, WREN);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));
}

/******************************************************************************
 * This function allows new values to be written to the Volatile Enhanced
 * Configuration Register. The Volatile Enhanced Configuration Register has the
 * following structure:
 *
 *    bit 7 | bit 6  |  bit 5  |  bit 4  |  bit 3  |  bit 2  |  bit 1  | bit 0
 * _________|________|_________|_________|_________|_________|_________|________
 * |  QUAD  |  DUAL  |/////////|    R    |   Acc   |         | OUTPUT  |        |
 * |--------|--------|---------|---------|---------|---------|---------|--------|
 *
 * Where:
 * QUAD = Quad Input Command
 * DUAL = Dual Input Command
 * Bit 5 = Reserved
 * R = Reset/Hold Disable
 * Acc = Accelerator pin enable
 * OUTPUT = Output Driver Strength
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DivRate is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	Mode is the mode on which the communication between Quad-SPI IP Core
 * 			and Flash Memory is made: can be Extended Mode, Dual Mode, Quad Mode.
 * @param	Data is the 8-bit unsigned value to write to the Volatile Enhanced
 * 			Configuration Register.
 *
 * @return	None.
 *
 * @note	This function is available for all three modes.
 *****************************************************************************/
void Write_Volatile_Enhanced_Configuration_Register(Xuint32 QuadSPI_Baseaddr,
		Xuint32 DivRate, Xuint32 Mode, u8 Data) {

	// Enable writing
	Write_Enable(QuadSPI_Baseaddr, DivRate, Mode);

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xBFFE0018&((DivRate<<17)|0xC001FFFF);
	end_command = 0x3FFE0018&((DivRate<<17)|0xC001FFFF);

	// Setting modes
	if(Mode == 2) { // cmd_mode, write_mode = 2
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x82);
	}
	else if(Mode == 1) { // cmd_mode, write_mode = 1
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x41);
	}
	else { // cmd_mode, write_mode = 0
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x00);
	}

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, WRVECR);
	 //write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Data into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, Data);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));
}

/******************************************************************************
 * This function reads Flash Memory's identification data.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DivRate is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	CurrentMode is the mode on which the Flash Memory was working on
 * 			the previous operation: can be Extended Mode, Dual Mode, Quad Mode.
 * @param	Data is the identification data read.
 *
 * @return	None.
 *
 * @note	This function is available only for Extended Mode.
 *****************************************************************************/
void Read_Identification(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, u8 *Data){

	// single command input
	if(CurrentMode) {
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0xFF);
	}

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xFFFE0054&((DivRate<<17)|0xC001FFFF);
	end_command = 0x7FFE0054&((DivRate<<17)|0xC001FFFF);

	// Setting modes (cmd_mode = 0, read_mode = 0)
	XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x00);

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, RDID);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));

	for(i=0; i<3; i++) {
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x01);
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);
		Data[i] = XIo_In32(QuadSPI_Baseaddr+DATA_OUT_REG);
	}
}

/******************************************************************************
 * This function reads Flash Memory's identification data.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DivRate is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	CurrentMode is the mode on which the Flash Memory was working on
 * 			the previous operation: can be Extended Mode, Dual Mode, Quad Mode.
 * @param	Data is the identification data read.
 *
 * @return	None.
 *
 * @note	This function is available only for Dual and Quad Modes.
 *****************************************************************************/
void Multiple_IO_Read_Identification(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 Mode, u8 *Data) {

	if(Mode == 2) { // quad command input
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0x7F);
	}
	else { // dual command input
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0xBF);
	}

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xFFFE0034&((DivRate<<17)|0xC001FFFF);
	end_command = 0x7FFE0034&((DivRate<<17)|0xC001FFFF);

	// Setting modes
	if(Mode == 2) { // cmd_mode = 2, read_mode = 2
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x22);
	}
	else { // cmd_mode = 1, read_mode = 1
		XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x11);
	}

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, MIORDID);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));

	for(i=0; i<3; i++) {
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x01);
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);
		Data[i] = XIo_In32(QuadSPI_Baseaddr+DATA_OUT_REG);
	}
}

/******************************************************************************
 * Reads data from the Flash memory at faster speed.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DIV_RATE is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	CURRENT_MODE is the mode on which the Flash Memory was working on
 * 			the previous operation: can be Extended Mode, Dual Mode, Quad Mode.
 * @param	ADDRESS is the start address from which the data will be read.
 * @param	NR_OF_BYTES is the number of bytes to read.
 * @param	NR_OF_DUMMY_CLKS is number of dummy clock cycles until data is ready
 * 			to be read.
 * @param	DATA is the data read.
 *
 * @return	None.
 *
 * @note	This function is available for all modes.
 *****************************************************************************/
void Quad_Output_Fast_Read(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 Address, Xuint32 NrOfBytes,
		Xuint32 NrOfDummyClks, u8 *Data) {

	//extended command input
	if(CurrentMode) {
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0xFF);
	}

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xBFFFFFF7&((DivRate<<17)|0xC001FFFF)&
			((NrOfDummyClks<<13)|0xFFFE1FFF)&((NrOfBytes<<4)|0xFFFFE00F);
	end_command = 0x3FFFFFF7&((DivRate<<17)|0xC001FFFF)&
			((NrOfDummyClks<<13)|0xFFFE1FFF)&((NrOfBytes<<4)|0xFFFFE00F);

	// Setting modes (cmd_mode, addr_mode = 0, read_mode = 2)
	XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x20);

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, QOFR);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 0 (most significant) into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, (Address>>16));
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 1 into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, (Address>>8));
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 2 (least significant) into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, Address);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));

	// Read Bytes from RxFIFO - data read from Flash Memory
	for(i=0; i<NrOfBytes; i++)
	{
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x01);
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);
		Data[i] = XIo_In32(QuadSPI_Baseaddr+DATA_OUT_REG);
	}
}

/******************************************************************************
 * This function writes a chunk of data into the memory at one go. This function
 * assumes that the memory to be programmed has been previously erased or that
 * bits are only changed from 1 to 0.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DIV_RATE is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	CURRENT_MODE is the mode on which the Flash Memory was working on
 * 			the previous operation: can be Extended Mode, Dual Mode, Quad Mode.
 * @param	ADDRESS is the start address from which the data will be written.
 * @param	NR_OF_BYTES is the number of bytes to write.
 * @param	DATA is the data to write.
 *
 * @return	None.
 *
 * @note	This function is available for all modes.
 *****************************************************************************/
void Quad_Input_Fast_Program(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 Address, Xuint32 NrOfBytes, u8 *Data) {

	//extended command input
	if(CurrentMode) {
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0xFF);
	}

	// Enable writing
	Write_Enable(QuadSPI_Baseaddr, DivRate, 0);

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xBFFE1FF9&((DivRate<<17)|0xC001FFFF)&
			((NrOfBytes<<4)|0xFFFFE00F);
	end_command = 0x3FFE1FF9&((DivRate<<17)|0xC001FFFF)&
			((NrOfBytes<<4)|0xFFFFE00F);

	// Set modes (cmd_mode, addr_mode = 0, write_mode = 2)
	XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x80);
	//XIo_Out32(QuadSPI_Baseaddr + MODES_REG, 0x00);

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, QIFP);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 0 (most significant) into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, (Address>>16));
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 1 into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, (Address>>8));
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 2 (least significant) into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, Address);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put data to write into TxFIFO
	for(i=0; i<NrOfBytes; i++) {
		XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, Data[i]);
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
		XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);
	}

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));

	do {
		Read_Status_Register(QuadSPI_Baseaddr, DivRate, 0, 1, &Status_Reg);
		Status_Reg &= 0x01;
	}
	while(Status_Reg);
}

/******************************************************************************
 * This function erases a subsector(4kb) of data from Flash Memory.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DivRate is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	CurrentMode is the mode on which the Flash Memory was working on
 * 			the previous operation: can be Extended Mode, Dual Mode, Quad Mode.
 * @param	Address is the start address from which the data will be erased.
 *
 * @return	None.
 *
 * @note	None.
 *****************************************************************************/
void Sector_Erase(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate, Xuint32 CurrentMode,
		Xuint32 Address) {

	if(CurrentMode) {
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0xFF);
	}

	// Enable writing/erasing
	Write_Enable(QuadSPI_Baseaddr, DivRate, 0);

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xFFFE0001&((DivRate<<17)|0xC001FFFF);
	end_command = 0x7FFE0001&((DivRate<<17)|0xC001FFFF);

	// cmd_mode, addr_mode = 0
	XIo_Out32 (QuadSPI_Baseaddr+MODES_REG, 0x00);

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, SE);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 0 (most significant) into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, (Address>>16));
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 1 into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, (Address>>8));
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Put Address 2 (least significant) into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, Address);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session end
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));

	do {
		Read_Status_Register(QuadSPI_Baseaddr, DivRate, 0, 1, &Status_Reg);
		Status_Reg &= 0x01;
	}
	while(Status_Reg);
}

/******************************************************************************
 * This function erases the Flash Memory.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DivRate is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	CurrentMode is the mode on which the Flash Memory was working on
 * 			the previous operation: can be Extended Mode, Dual Mode, Quad Mode.
 *
 * @return	None.
 *
 * @note	None.
 *****************************************************************************/
void Bulk_Erase(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate, Xuint32 CurrentMode) {

	if(CurrentMode) {
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0xFF); // single command input
	}

	// Enable writing/erasing
	Write_Enable(QuadSPI_Baseaddr, DivRate, 0);

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xFFFE0000&((DivRate<<17)|0xC001FFFF);
	end_command = 0x7FFE0000&((DivRate<<17)|0xC001FFFF);

	// cmd_mode, addr_mode = 0
	XIo_Out32 (QuadSPI_Baseaddr+MODES_REG, 0x00);

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, BE);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));

	do {
		Read_Status_Register(QuadSPI_Baseaddr, DivRate, 0, 1, &Status_Reg);
		Status_Reg &= 0x01;
	}
	while(Status_Reg);
}

/******************************************************************************
 * Read the Status Register. Its structure is the following:
 *
 *   bit 7 = Status register write protect
 *   bit 6 = Block protect 3
 *   bit 5 = Top/Bottom bit
 *   bit 4 = Block protect 2
 *   bit 3 = Block protect 1
 *   bit 2 = Block protect 0
 *   bit 1 = Write enable latch
 *   bit 0 = Write in progress
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	DIV_RATE is the divide rate of the System Clock, for generating
 * 			Serial Clock for the Flash memory.
 * @param	CURRENT_MODE is the mode on which the Flash Memory was working on
 * 			the previous operation: can be Extended Mode, Dual Mode, Quad Mode.
 * @param	NR_OF_BYTES is the number of bytes to read.
 * @param	Data is the data read.
 *
 * @return	None.
 *
 * @note	This function is available for all modes.
 *****************************************************************************/
void Read_Status_Register(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 NrOfBytes, u8 *Data) {

	if(CurrentMode) {
		Write_Volatile_Enhanced_Configuration_Register(QuadSPI_Baseaddr,
				DivRate, CurrentMode, 0xFF);
	}

	// Reseting FSM's and FIFO's
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+RESET_REG, 0x00);

	start_command = 0xFFFE1FF4&((DivRate<<17)|0xC001FFFF)&((1<<4)|0xFFFFE00F);
	end_command = 0x7FFE1FF4&((DivRate<<17)|0xC001FFFF)&((1<<4)|0xFFFFE00F);

	// cmd_mode, read_mode = 0
	XIo_Out32(QuadSPI_Baseaddr+MODES_REG, 0x00);

	// Put instruction command into TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+DATA_IN_REG, RDSR);
	//write enable on TxFIFO
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x02);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);

	// Start a cycle session
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, start_command);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_CYCLE_REG, end_command);

	// Wait until session ends
	do {
		Status = XIo_In32(QuadSPI_Baseaddr+STATUS_REG);
	}
	while(!((Status>>4)&0x01));

	// Read Bytes from RxFIFO - data read from Flash memory
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x01);
	XIo_Out32(QuadSPI_Baseaddr+CONTROL_FIFO_REG, 0x00);
	Data[0] = XIo_In32(QuadSPI_Baseaddr+DATA_OUT_REG);
}

/******************************************************************************
 * This function verifies in which mode (Extended, Dual or Quad) is the Flash
 * memory currently set.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 *
 * @return	0x00 - Extended Mode (or Single)
 * 			0x02 - Dual Mode
 * 			0x04 - Quad Mode
 * 			0xFF - Error
 *
 * @note	None.
 *****************************************************************************/
Xuint32 Check_Initial_Mode(Xuint32 QuadSPI_Baseaddr) {

	u8 Data[3];

	// Check Read ID in MODE 0
	Read_Identification(QuadSPI_Baseaddr, 4, 0, Data);
	//xil_printf("\nDone reading single mode id:\nmanufact: 0x%x\ntype: "
	//		"0x%x\ncapacity: 0x%x", Data[0], Data[1], Data[2]);

	if(Data[0] == 0x20) {
		if(verbose) xil_printf("\nCurrent mode: extended");
		return 0x00;
	}
	else {
		// Check Read ID in MODE 1
		Multiple_IO_Read_Identification(QuadSPI_Baseaddr, 4, 1, 1, Data);
		//xil_printf("\nDone reading dual mode id:\nmanufact: 0x%x\ntype: "
		//		"0x%x\ncapacity: 0x%x", Data[0], Data[1], Data[2]);

		if(Data[0] == 0x20) {
			if (verbose) xil_printf("\nCurrent mode: dual");
			return 0x01;
		}
		else {
			// Check Read ID in MODE 2
			Multiple_IO_Read_Identification(QuadSPI_Baseaddr, 4, 2, 2, Data);
			//xil_printf("\nDone reading quad mode id:\nmanufact: 0x%x\ntype: "
			//		"0x%x\ncapacity: 0x%x", Data[0], Data[1], Data[2]);

			if(Data[0] == 0x20) {
				if (verbose) xil_printf("\nCurrent mode: quad");
				return 0x02;
			}
			else {
				if (verbose) xil_printf("\nError reading idcode");
				return 0xFF;
			}
		}
	}
}

/******************************************************************************
 * This function checks if a certain sector from the Flash memory is empty by
 * reading its contents and comparing every Byte with 0xFF (default value after
 * an Erase operation).
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 * @param	ADDRESS is the address of the block to be checked.
 *
 * @return	0x00 - The Flash memory is blank
 * 			0x?? - Number of non-blank Bytes
 * 			0xFF - Error
 *
 * @note	None.
 *****************************************************************************/
Xuint32 Blank_Check_Sector(Xuint32 QuadSPI_Baseaddr, Xuint32 Address) {

	Xuint32 i;
	Xuint32 j;
	Xuint32 z = 0;
	Xuint32 Non_Blank_Bytes = 0;
	Xuint32 Current_Mode;
	u8 *Data;

	Data = (u8 *)malloc(256);
	Address &= 0xFF0000;

	// Check initial mode (current mode)
	Current_Mode = Check_Initial_Mode(QuadSPI_Baseaddr);

	if(Current_Mode == 0xFF) {
		// Check again
		Current_Mode = Check_Initial_Mode(QuadSPI_Baseaddr);

		if(Current_Mode == 0xFF) {
			// Check again
			Current_Mode = Check_Initial_Mode(QuadSPI_Baseaddr);
		}
	}

	for(i=Address; i<(Address+65536); i+=256) {
		// Read every Byte from memory
		Quad_Output_Fast_Read(QuadSPI_Baseaddr, 2, Current_Mode, i, 256, 8, Data);
		Current_Mode = 0;

		if(verbose) {
			if(i==z) {
				if(verbose) xil_printf("\n Data[0x%x]: 0x%x", i, Data[0]);
				z = z + 0x1000;
			}
		}

		// Compare every Byte read: if it is not blank erase that sector
		for(j=0; j<256; j++) {
			if(Data[j] != 0xFF) {
				Non_Blank_Bytes++;
			}
		}
	}

	if(Non_Blank_Bytes != 0) {
		free(Data);
		return (Non_Blank_Bytes);
	}

	free(Data);
	return 0x00;
}

/******************************************************************************
 * This function does a basic test of the Flash memory by writing consequently:
 * 0x00, 0x01, 0x04, 0x08, 0xFF and then read it back. It then compares read
 * data with the written one.
 *
 * @param	QuadSPI_Baseaddr is the base address for the Quad-SPI IP Core.
 *
 * @return	0xFF - memory not passed the test
 * 			0x00 - memory passed the test
 *
 * @note	None.
 *****************************************************************************/
Xuint32 Quad_SPI_Flash_Test(Xuint32 QuadSPI_Baseaddr) {

	u8 dataToWrite[6];
	u8 dataRead[6];
	Xuint32 Current_Mode;
	Xuint32 startAddr = 0x00;

	// Assign the control Bytes (to be written into Flash)
	dataToWrite[0] = 0xFF;
	dataToWrite[1] = 0xEE;
	dataToWrite[2] = 0x01;
	dataToWrite[3] = 0x02;
	dataToWrite[4] = 0x04;
	dataToWrite[5] = 0x08;

	// Check initial mode (current mode)
	Current_Mode = Check_Initial_Mode(QuadSPI_Baseaddr);

	// Check if IDCODE is correct
	if(Current_Mode == 0xFF) {

		// Check again
		Current_Mode = Check_Initial_Mode(QuadSPI_Baseaddr);

		if(Current_Mode == 0xFF) {

			// Check again
			Current_Mode = Check_Initial_Mode(QuadSPI_Baseaddr);

			if(Current_Mode == 0xFF) {
				return 0xFF; // "memory not found"
			}
		}
	}

	// Erase the memory to ensure a proper writing
	Sector_Erase(QuadSPI_Baseaddr, 4, Current_Mode, startAddr);

	// Write to memory in BY-4 mode: 0x00, 0xFF, 0x01, 0x02, 0x04, 0x08
	Quad_Input_Fast_Program(QuadSPI_Baseaddr, 4, 0, startAddr, 6, dataToWrite);

	// Read data from the memory
	Quad_Output_Fast_Read(QuadSPI_Baseaddr, 4, 0, startAddr, 6, 8, dataRead);

	// Erase written data
	Sector_Erase(QuadSPI_Baseaddr, 4, 0, startAddr);

	/*if(verbose) {
		xil_printf("\nData_Read[0]: 0x%x", dataRead[0]);
		xil_printf("\nData_Read[1]: 0x%x", dataRead[1]);
		xil_printf("\nData_Read[2]: 0x%x", dataRead[2]);
		xil_printf("\nData_Read[3]: 0x%x", dataRead[3]);
		xil_printf("\nData_Read[4]: 0x%x", dataRead[4]);
		xil_printf("\nData_Read[5]: 0x%x", dataRead[5]);
	}*/

	// Check if data read is the same as the written one
	if ((dataRead[0] == dataToWrite[0]) &&
		(dataRead[1] == dataToWrite[1]) &&
		(dataRead[2] == dataToWrite[2]) &&
		(dataRead[3] == dataToWrite[3]) &&
		(dataRead[4] == dataToWrite[4]) &&
		(dataRead[5] == dataToWrite[5])) {
		return 0x00; // "memory passed the test"
	}
	else {
		return 0xFF; // "read from memory failed"
	}
}
