/******************************************************************************
 * @file xquad_spi.h
 *
 * This header file contains identifiers and driver functions (or macros) that
 * can be used to access the device.  The user should refer to the hardware
 * device specification for more details of the device operation.
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date     Changes
 * ----- ---- -------- -----------------------------------------------
 * 1.00a nagy 04/04/11 First release
 *
 * </pre>
 *
 *****************************************************************************/

#ifndef XQUAD__SPI_H	/* prevent circular inclusions */
#define XQUAD__SPI_H	/* by using protection macros */

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files *********************************/

#include <stdio.h>
#include <stdlib.h>
#include <xio.h>
#include <xparameters.h>

/**************************** Type Definitions *******************************/

/* @name Register Offsets
 *
 * Offsets of the registers from the serial Flash controller
 * @{
 */

enum {
	RESET_REG				= 0x00,
	STATUS_REG				= 0x04,
	OCCUPANCY_RXFIFO_REG	= 0x08,
	OCCUPANCY_TXFIFO_REG	= 0x0C,
	CONTROL_CYCLE_REG		= 0x10,
	DATA_IN_REG				= 0x14,
	DATA_OUT_REG			= 0x18,
	CONTROL_FIFO_REG		= 0x1C,
	MODES_REG				= 0x20
};

/* @name Commands
 *
 * Commands for Flash operations
 * @{
 */

enum {
	RDID 		= 0x9F, /* Read Identification */
	MIORDID 	= 0xAF, /* Multiple I/O Read Identification */
	WREN 		= 0x06, /* Write Enable */
	SE			= 0xD8, /* Sector Erase */
	BE			= 0xC7, /* Bulk Erase */
	RDSR 		= 0x05, /* Read Status Register */
	WRVECR 		= 0x61, /* Write Volatile Enhanced Configuration Register */
	QOFR 		= 0x6B, /* Quad Output Fast Read */
	QIFP 		= 0x32  /* Quad Input Fast Read */
};

/************************** Function Prototypes ******************************/

void Write_Enable(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate, Xuint32 Mode);
void Write_Volatile_Enhanced_Configuration_Register(Xuint32 QuadSPI_Baseaddr,
		Xuint32 DivRate, Xuint32 Mode, u8 Data);
void Read_Identification(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, u8 *Data);
void Multiple_IO_Read_Identification(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 Mode, u8 *Data);
void Quad_Output_Fast_Read(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 Address, Xuint32 NrOfBytes,
		Xuint32 NrOfDummyClks, u8 *Data);
void Quad_Input_Fast_Program(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 Address, Xuint32 NrOfBytes, u8 *Data);
void Sector_Erase(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate, Xuint32 CurrentMode,
		Xuint32 Address);
void Bulk_Erase(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate, Xuint32 CurrentMode);
void Read_Status_Register(Xuint32 QuadSPI_Baseaddr, Xuint32 DivRate,
		Xuint32 CurrentMode, Xuint32 NrOfBytes, u8 *Data);
Xuint32 Check_Initial_Mode(Xuint32 QuadSPI_Baseaddr);
Xuint32 Blank_Check_Sector(Xuint32 QuadSPI_Baseaddr, Xuint32 Address);
Xuint32 Quad_SPI_Flash_Test (Xuint32 QuadSPI_Baseaddr);
void usleep(unsigned int useconds);


#ifdef __cplusplus
}
#endif

#endif /* end of protection macro */

